/*  
 *  Aggiunta della classe Point3D, che eredita da Point.
 *  Rimossa la parte di Line.
 * compilare: 
 * -> g++ -o main4 main4.cpp 
 * eseguire:
 * -> ./main4
 */

#include <cmath>
#include <iostream>
#include <string>

using namespace std;

/* dichiarazione classe Point */
class Point {
public:
  Point();
  Point(string name);
  Point(string name, double x, double y);

  void SetX(double x) { fX = x; }
  void SetY(double y) { fY = y; }
  double GetX() { return fX; }
  double GetY() { return fY; }
  double Magnitude();
  // la funzione Print diventa virtual, per fare in modo che puntatori di tipo
  // "Point", ma che puntano in realta' ad oggetti di tipo "Point3D", chiamino automaticamente la Print() implementata in Point3D
  virtual void Print();

// le variabili membro passano da private a protected per garantirne
// l'accesso alle classi figlie (come Point3D). Oggetti e funzioni
// esterni a Point e Point3D non possono comunque accedervi.
protected:
  double fX;
  double fY;
  string fName;

};

/* implementazione classe Point */
Point::Point() 
{
  fX = 0;
  fY = 0;
  fName = "Unknown";
}

Point::Point(string name) 
{
  fX = 0;
  fY = 0;
  fName = name;
}

Point::Point(string name, double x, double y)
{
  fName = name;
  fX = x;
  fY = y;
}

double Point::Magnitude()
{
    return sqrt(fX*fX+fY*fY); 
}

void Point::Print() 
{
  cout<<fName<<": fX = "<<fX<<" fY = "<<fY<<endl;
}


/* dichiarazione della classe Point3D (con costruttore vuoto) */
class Point3D : public Point
{
    public:
        Point3D() {} ;
        void Print() {
            cout<<fX<<" "<<fY<<" "<<fZ<<endl;
        }

        void SetZ(double z) { fZ = z; }

    private:
        double fZ;

};


/* funzione main() */
int main() {
    Point A("PuntoA"),B("PuntoB");
//    Point C("PuntoC",19,13.2),D;
//    A.Print(); 
    A.SetX(3.2);
    A.SetY(1.2);
    A.Print();
    B.SetX(5.2);
    B.SetY(7.3);
    B.Print();
//    C.Print();
//    D.Print();

/*    
    double magA = A.Magnitude();
    double magB = B.Magnitude();

    cout<<"MagA "<<magA<<endl;
    cout<<"MagB "<<magB<<endl;
    */

    Point3D D3;
    D3.SetX(1);
    D3.SetY(2);
    D3.SetZ(3);
    // chiama la print di Point3D (anche se Point::Print() non era virtual)
    D3.Print();

    // ripasso puntatori
    Point3D* p3D = &D3;
    p3D->Print();
    (*p3D).Print();

    // Anche se il puntatore e' di tipo Point, grazie al virtual 
    // nella classe madre, viene chiamato il Print della figlia.
    // Senza virtual, sarebbe stato chiamato il Print della madre.
    Point* p2D = &D3;
    p2D->Print();


    // Creare un oggetto nuovo e avere direttamente un puntatore
    // ad esso
    Point3D* pnD3 = new Point3D();
    pnD3->SetX(3);
    pnD3->SetY(5);
    pnD3->SetZ(7);

    return 0;
}
